//=============================================================================
// DawnsBigRock.
//
// Faster implementation of BigRock.
//
// Version: 1.0
// $DateTime: Wed, 07/13/2005 11:38:02 am$
// Author: Dawn Knight
//
// UT Version Coded/Tested: 4.36.
//=============================================================================
class DawnsBigRock extends BigRock;

function TakeDamage( int NDamage, Pawn instigatedBy,
				Vector hitlocation, Vector momentum, name damageType)
{

	Velocity += Momentum /(DrawScale * 10);
	if (Physics == PHYS_None )
	{
		SetPhysics(PHYS_Falling);
		Velocity.Z += 0.4 * VSize(Momentum);
	}
	SpawnChunks1();
}

final function SpawnChunks1()
{
	local int    NumChunks,i;
	local DawnsBigRock   TempRock;
	local float CScale;

	if ( DrawScale < 3 + FRand() )
		return;

	NumChunks = 1 + Rand(2);
	CScale = 0.4;
	if ( DrawScale < 6 )
	{
		NumChunks = 0;
	}
	speed = VSize(Velocity);
	for ( i=0; i<NumChunks; i++ )
	{
		TempRock = Spawn(class'DawnsBigRock');
		if (TempRock != None )
			TempRock.InitFrag1(self, CScale);
	}
	InitFrag1(self, 0.35);
}

final function InitFrag1(DawnsBigRock myParent, float FScale)
{
	local rotator newRot;

	RotationRate = RotRand();
	FScale *= (0.5 + FRand());
	DrawScale = FScale * myParent.DrawScale;
	if ( DrawScale <= 2 )
		SetCollisionSize(0,0);
	else
		SetCollisionSize(CollisionRadius * DrawScale/Default.DrawScale, CollisionHeight * DrawScale/Default.DrawScale);

	Velocity = Normal(VRand() + myParent.Velocity/myParent.speed)
				* (myParent.speed * (0.4 + 0.3 * (FRand() + FRand())));
}

auto state Flying
{
	function ProcessTouch (Actor Other, Vector HitLocation)
	{
		local int hitdamage;

		if ( Other == instigator )
			return;
		PlaySound(ImpactSound, SLOT_Interact, DrawScale/10);

		if ( !Other.IsA('BigRock') )
		{
			Hitdamage = Damage * 0.0003 * (DrawScale) * speed;
			if ( (HitDamage > 6) && (speed > 150) )
				Other.TakeDamage(hitdamage, instigator,HitLocation,
					(35000.0 * Normal(Velocity)), 'crushed' );
		}
	}
	simulated function HitWall (vector HitNormal, actor Wall)
	{
		local vector RealHitNormal;

		speed = VSize(velocity);
		MakeSound();
		if ( speed < 9 )
			Destroy();
		else
		{
			SetPhysics(PHYS_Falling);
			RealHitNormal = HitNormal;
			if ( FRand() < 0.5 )
				RotationRate.Pitch = Max(RotationRate.Pitch, 100000);
			else
				RotationRate.Roll = Max(RotationRate.Roll, 100000);
			HitNormal = Normal(HitNormal + 0.5 * VRand());
			if ( (RealHitNormal Dot HitNormal) < 0 )
				HitNormal.Z *= -0.7;
			Velocity = 0.7 * (Velocity - 2 * HitNormal * (Velocity Dot HitNormal));
			DesiredRotation = rotator(HitNormal);
			if ( (speed > 150) && (FRand() * 30 < DrawScale) )
				SpawnChunks1();
		}
	}

Begin:
	Sleep(2.0);
	SetPhysics(PHYS_Falling);
}

state Sitting
{
Begin:
	SetPhysics(PHYS_Falling);
	Sleep(DrawScale * 0.025);
	Destroy();
}

defaultproperties
{
     speed=1600.000000
     MaxSpeed=2000.000000
     LifeSpan=12.000000
}
