//=============================================================================
// DawnsDelayedCharge for UT.  By Dawn Knight
// Version 1.02
// $DateTime: Tue, 06/28/2005 02:53:20 am$
//
// Timed fuse explosive charge.
//
// Sets FuseTime (delay), ExplosionSize, and Damage
//
// IMPORTANT FOR MAPPERS!!!
// There is a subclass of Assault named LeagueAS140 running on many Assault
// servers that replaces any FortStandard whose bTriggerOnly is True with a
// subclass of TeamTrigger named FortStandardTrigger.  Unfortunately,
// bCollideActors is NOT copied over in the replacement and causes the
// FortStandardTrigger to be triggerable by Touch()ing it, which is NOT okay!
// Solution:  Set this DawnsDelayedCharge's associated FortStandard.bTriggerOnly
// to False and set FortStandard.bProjTarget to False. As a result of these 2
// flag changes, the associated FortStandard will not get replaced and only this
// DawnsDelayedCharge will be able to Trigger the FortStandard.
// Setting bAutoAdjFortStd to True in this DawnsDelayedCharge will AUTOMATICALLY
// perform these adjustments ingame for you.
//
//=============================================================================
class DawnsDelayedCharge extends Decoration;

#exec obj load file=..\Textures\Starship.utx package=Starship
#exec obj load file=..\Textures\Crypt2.utx package=Crypt2
#exec obj load file=..\Sounds\LadderSounds.uax
#exec texture IMPORT name=OliveDrab FILE=Textures\OliveDrab1.pcx

#exec texture IMPORT name=LEDNum15 FILE=Textures\LEDNum15.pcx
#exec texture IMPORT name=LEDNum14 FILE=Textures\LEDNum14.pcx
#exec texture IMPORT name=LEDNum13 FILE=Textures\LEDNum13.pcx
#exec texture IMPORT name=LEDNum12 FILE=Textures\LEDNum12.pcx
#exec texture IMPORT name=LEDNum11 FILE=Textures\LEDNum11.pcx
#exec texture IMPORT name=LEDNum10 FILE=Textures\LEDNum10.pcx
#exec texture IMPORT name=LEDNum09 FILE=Textures\LEDNum09.pcx
#exec texture IMPORT name=LEDNum08 FILE=Textures\LEDNum08.pcx
#exec texture IMPORT name=LEDNum07 FILE=Textures\LEDNum07.pcx
#exec texture IMPORT name=LEDNum06 FILE=Textures\LEDNum06.pcx
#exec texture IMPORT name=LEDNum05 FILE=Textures\LEDNum05.pcx
#exec texture IMPORT name=LEDNum04 FILE=Textures\LEDNum04.pcx
#exec texture IMPORT name=LEDNum03 FILE=Textures\LEDNum03.pcx
#exec texture IMPORT name=LEDNum02 FILE=Textures\LEDNum02.pcx
#exec texture IMPORT name=LEDNum01 FILE=Textures\LEDNum01.pcx
#exec texture IMPORT name=LEDNum00 FILE=Textures\LEDNum00.pcx

#exec mesh import mesh=DawnsDelayedChargeM anivfile=Models\DawnsDelayedChargeM_a.3d datafile=Models\DawnsDelayedChargeM_d.3d x=0 y=0 z=0 mlod=0
#exec mesh origin mesh=DawnsDelayedChargeM x=0 y=0 z=0
#exec mesh sequence mesh=DawnsDelayedChargeM seq=All startframe=0 numframes=1

#exec meshmap new meshmap=DawnsDelayedChargeM mesh=DawnsDelayedChargeM
#exec meshmap scale meshmap=DawnsDelayedChargeM x=0.06010 y=0.06010 z=0.12020

var		bool 		bFused, bExplode, bDestroy;
var() bool    bAutoAdjFortStd;
var() int 		Fusetime; // Fuse adjustable from 1 to 15 seconds
var() sound		EffectSound1;
var		texture FuseCount[16], LastDisplay; // LEDNumerals from 00 to 15
var		int 		CurrentCount;

// Damage attributes.
var() float		Damage;
var() int			MomentumTransfer; // Momentum imparted by impacting projectile.
var() name		MyDamageType;
var() float		ExplosionSize;

// Human readable triggering message.
var() localized string Message;

var		pawn		Attacker;  // Player on Attacker Team that succesfully explodes charge
var   FortStandard FortStd;
var   Assault AG;

replication
{
	reliable if( Role==ROLE_Authority )
		bExplode;
}

function PreBeginPlay()
{
	local Actor A;
	local FortStandard F;

	super.PreBeginPlay();
	FuseTime = Clamp(FuseTime, 1, 15);
	CurrentCount = FuseTime;
	MultiSkins[4] = FuseCount[FuseTime];
	LastDisplay = MultiSkins[4];
	foreach AllActors( class'FortStandard', F, event )
	{
		FortStd = F;
		break;
	}
	if ( bAutoAdjFortStd )
		if( event != '' )
			foreach AllActors( class 'Actor', A, Event )
			{
				if ( FortStandard(A) != None )
				{
					A.SetCollision(False, False, False);
					FortStandard(A).bTriggerOnly = False;
					FortStandard(A).bProjTarget = False;
				}
				else
				if ( TeamTrigger(A) != None && A.IsA('FortStandardTrigger') )
				{
					A.SetCollision(False, False, False);
				}
			}
}

function PostBeginPlay()
{
	Disable('Tick'); // Tick still running on Clients!
	if ( !Level.Game.IsA('Assault') )
	{
		Destroy();
		return;
	}
	AG = Assault(Level.Game);
}

final function DeactivateFort()
{
	local bool bFound;
	local int i;

	if ( !FortStd.bFinalFort )
	{
		for ( i=0; i<(AG.numForts - 1); i++ )
		{
			if ( AG.Fort[i] == FortStd )
				bFound = true;
			if ( bFound )
				AG.Fort[i] = AG.Fort[i+1];
		}
		AG.Fort[AG.numForts] = None;
		AG.numForts--;
	}
}

final function ActivateFort()
{
	local bool bFound;
	local int i;

	for ( i=0; i<AG.numForts; i++ )
	{
		if ( AG.Fort[i] == FortStd )
			bFound = True;
	}
	if ( !bFound )
	{
		AG.Fort[AG.numForts] = FortStd;
		AG.numForts++;
	}
}

function Touch( actor Other)
{
	if ( Other.bIsPawn && Pawn(Other).bIsPlayer )
	{
		if ( Pawn(Other).PlayerReplicationInfo.Team == Assault(Level.Game).Defender.TeamIndex )
		{ // Defender
			if ( bFused )
			{
				bHidden = true;
				bFused = false;
				CurrentCount = FuseTime;
				MultiSkins[4] = FuseCount[FuseTime];
//				ActivateFort();
			}
		}
		else
		{  // Attacker
			if ( !bFused )
			{
				bHidden = false;
				bFused = true;
				CurrentCount = FuseTime;
				SetTimer(1.0,False);
				Attacker = Pawn(Other);
//				DeactivateFort();
//				if ( Bot(Other) != None )
//					Bot(Other).GotoState('Retreating');
			}
		}
	}
}

function BlowUp()
{
	local actor A;

	HurtRadius(Damage,220.0, MyDamageType, MomentumTransfer, Location );
//	MakeNoise(1.0);

	// Broadcast a Trigger message to all matching actors.
	if( event != '' )
		foreach AllActors( class 'Actor', A, Event )
			A.Trigger( Attacker, Attacker.Instigator );

	if ( Attacker.SpecialGoal == self)
		Attacker.SpecialGoal = None;

	if( Message != "" && Attacker != none )
		// Send a string message to the Attacker.
		Attacker.Instigator.ClientMessage( Message );


}

simulated function Explode()
{
	local UT_SpriteBallExplosion s;

	if ( Level.NetMode != NM_DedicatedServer )
	{
		bHidden = true;
		s = Spawn(class'UT_SpriteBallExplosion');
		s.RemoteRole = ROLE_None;
		s.DrawScale = ExplosionSize;
	}
	if ( ROLE == ROLE_Authority )
		BlowUp();
}

simulated function Tick(float DeltaTime)
{
	// Only running on Net Clients.
	if ( bExplode ) // Yes, server says to explode.
	{
		Explode();
		Destroy();
	}
	else // No, don't explode yet, keep checking.
	if ( Multiskins[4] != LastDisplay )
	{
		PlaySound(EffectSound1, , 3.0, , 1600);
		LastDisplay = MultiSkins[4];
	}
}

function Timer()
{
	// Does not run on Clients.
	if ( bDestroy ) // Client completed Explode() and Destroy().
	{
		RemoteRole = ROLE_DumbProxy;
		Destroy();  // Destroy on Authority, too.
		return;
	}

	if ( !bFused || Level.Game.bGameEnded ) return;

	SetTimer(1.0,false);

	if ( CurrentCount < 0 )
	{
		disable('Touch'); // Too late!
		bExplode = true;
		Explode();
		bDestroy = true;
		return;
	}

	MultiSkins[4] = FuseCount[CurrentCount];
	PlaySound(EffectSound1, , 3.0, , 1600);
	CurrentCount--;
}

defaultproperties
{
     FuseTime=15
     DrawType=DT_Mesh
     Mesh=Mesh'DawnsDelayedChargeM'
     bHidden=True
     bCollideWhenPlacing=True
     bCollideActors=True
     bCollideWorld=True
     bBlockActors=False
     bBlockPlayers=False
     bStatic=False
     bStasis=True
     CollisionRadius=9.00000
     CollisionHeight=16.00000
     Damage=75.000000
     EffectSound1=Sound'LadderSounds.lcursorMove'
     ExplosionSize=3.00000
     MomentumTransfer=80000
     Message="Charge Detonated!"
     MyDamageType=GrenadeDeath
     MultiSkins(0)=Texture'Starship.Base.Grey'
     MultiSkins(1)=Texture'Crypt2.Pillar.cBlodBm3'
     MultiSkins(2)=Texture'OliveDrab'
     MultiSkins(3)=Texture'Starship.Base.C_bs4'
     MultiSkins(4)=Texture'LEDNum15'
     FuseCount(0)=texture'LEDNum00'
     FuseCount(1)=texture'LEDNum01'
     FuseCount(2)=texture'LEDNum02'
     FuseCount(3)=texture'LEDNum03'
     FuseCount(4)=texture'LEDNum04'
     FuseCount(5)=texture'LEDNum05'
     FuseCount(6)=texture'LEDNum06'
     FuseCount(7)=texture'LEDNum07'
     FuseCount(8)=texture'LEDNum08'
     FuseCount(9)=texture'LEDNum09'
     FuseCount(10)=texture'LEDNum10'
     FuseCount(11)=texture'LEDNum11'
     FuseCount(12)=texture'LEDNum12'
     FuseCount(13)=texture'LEDNum13'
     FuseCount(14)=texture'LEDNum14'
     FuseCount(15)=texture'LEDNum15'
     RemoteRole=ROLE_SimulatedProxy
     NetPriority=2.500000
}
