//=============================================================================
// DawnsEndSeqConsole.
//
// Replaces Player's Console during EndSequence and traps ESC keypresses in
// SinglePlayer (NetMode = NM_StandAlone) games only.
//
// Version: 1.0
// $DateTime: Sat, 07/02/2005 10:16:49 am$
// Author: Dawn Knight
// Based on CSHPConsole code by Dr. Sin.  Used with permission.
//
// UT Version Coded/Tested: 4.36 and compatible with UTPureRC7G.
//=============================================================================
class DawnsEndSeqConsole extends UTConsole;

var UTConsole            OldConsole;
var DawnsEnhancedEndSeq  MyDEES;
var int                  MyState; // 0 = global 1 = UWindow 2 = Typing.

event PreRender( canvas Canvas);

event PostRender( canvas C )
{
	local int YStart, YEnd, Y, I, J, Line, iLine;

	if ( MyState == 1 )
		WindowPostRender(c);
	else
	{
		if ( bNoDrawWorld )
		{
			C.SetPos(0,0);
			C.DrawPattern( Texture'Border', C.ClipX, C.ClipY, 1.0 );
		}
		if ( bTimeDemo )
		{
			TimeDemoCalc();
			TimeDemoRender( C );
		}
		DrawLevelAction( C );
		if ( ConsoleLines > 0 )
		{
			C.SetOrigin(0.0, ConsoleLines - FrameY*0.6);
			C.SetPos(0.0, 0.0);
			C.DrawTile( ConBackground, FrameX, FrameY*0.6, C.CurX, C.CurY, FrameX, FrameY );
		}
		if ( BorderLines > 0 || BorderPixels > 0 )
		{
			YStart	 = BorderLines + ConsoleLines;
			YEnd	 = FrameY - BorderLines;
			if ( BorderLines > 0 )
			{
				C.SetOrigin(0.0, 0.0);
				C.SetPos(0.0, 0.0);
				C.DrawPattern( Border, FrameX, BorderLines, 1.0 );
				C.SetPos(0.0, YEnd);
				C.DrawPattern( Border, FrameX, BorderLines, 1.0 );
			}
			if ( BorderPixels > 0 )
			{
				C.SetOrigin(0.0, 0.0);
				C.SetPos(0.0, YStart);
				C.DrawPattern( Border, BorderPixels, YEnd - YStart, 1.0 );
				C.SetPos( FrameX - BorderPixels, YStart );
				C.DrawPattern( Border, BorderPixels, YEnd - YStart, 1.0 );
			}
		}
		C.SetOrigin(0.0, 0.0);
		if ( ConsoleLines > 0 )
			DrawConsoleView( C );
		else
			DrawSingleView( C );
		//Global:
		//UT Console:
		if ( bShowSpeech || bShowMessage )
			RenderUWindow(C);
	}
	Viewport.Actor.bDelayedCommand = false;
}

final function WindowPostRender(canvas canvas)
{ //UWindow.postrender

	if ( bTimeDemo )
	{
		TimeDemoCalc();
		TimeDemoRender( Canvas );
	}
	if ( Root != None )
		Root.bUWindowActive = True;
	RenderUWindow( Canvas );
}

event bool KeyType( EInputKey Key )
{
	if ( MyState == 0 )
		return false;

	if ( MyState == 1 )
	{
		if ( Root == None )
			return false;
		Root.WindowEvent(WM_KeyType, None, MouseX, MouseY, Key);
		return True;
	}
	if ( bNoStuff )
	{
		bNoStuff = false;
		return true;
	}
	if ( Key>=0x20 && Key<0x100 && Key!=Asc("~") && Key!=Asc("`") )
	{
		TypedStr = TypedStr $ Chr(Key);
		Scrollback=0;
		return true;
	}
}

event NotifyLevelChange()
{
	// Restore OldConsole
	SetOldConsole();
	OldConsole.NotifyLevelChange();
}

final function InitNewConsole()
{
	local int i;

	ConsoleWindow = OldConsole.ConsoleWindow;
	viewport=OldConsole.viewport;
	Root = OldConsole.Root;
	Root.console=self;
	SpeechWindow = OldConsole.SpeechWindow;
	MessageWindow = OldConsole.MessageWindow;
	bTyping = OldConsole.bTyping;
	TypedStr = OldConsole.TypedStr;
	bShowSpeech = OldConsole.bShowSpeech;
	bCreatedRoot = OldConsole.bCreatedRoot;
	bshowconsole = OldConsole.bshowconsole;
	MouseX= OldConsole.MouseX;
	MouseY= OldConsole.MouseY;
	bUWindowType= OldConsole.bUWindowType;
	bUWindowActive=OldConsole.bUWindowActive;
	bLocked=OldConsole.bLocked;
	bQuickKeyEnable=OldConsole.bQuickKeyEnable;
	showdesktop=OldConsole.showdesktop;
	bTimeDemo=OldConsole.bTimeDemo;
	bSaveTimeDemoToFile=OldConsole.bSaveTimeDemoToFile;
	HistoryCur = OldConsole.HistoryCur;
	HistoryTop = OldConsole.HistoryTop;
	for (i = 0; i < MaxHistory; i++)
	    History[i] = OldConsole.History[i];

	//States:
	switch (OldConsole.GetStateName())
	{
		case 'UWindow':
			MyState = 1;
			break;
		case 'Typing':
			MyState = 2;
			break;
		default:
			MyState = 0;
	}
}

final function SetOldConsole()
{
	local int i;
	//return to old console.
	OldConsole.ConsoleWindow = ConsoleWindow;
	OldConsole.Root = Root;
	OldConsole.Root.Console=OldConsole;
	OldConsole.SpeechWindow = SpeechWindow;
	OldConsole.MessageWindow = MessageWindow;
	OldConsole.bTyping = bTyping;
	OldConsole.TypedStr = TypedStr;
	OldConsole.bShowSpeech = bShowSpeech;
	OldConsole.bCreatedRoot = bCreatedRoot;
	OldConsole.bshowconsole = bshowconsole;
	OldConsole.MouseX= MouseX;
	OldConsole.MouseY= MouseY;
	OldConsole.bUWindowType= bUWindowType;
	OldConsole.bUWindowActive=bUWindowActive;
	OldConsole.bLocked=bLocked;
	OldConsole.bQuickKeyEnable=bQuickKeyEnable;
	OldConsole.showdesktop=showdesktop;
	OldConsole.bTimeDemo=bTimeDemo;
	OldConsole.bSaveTimeDemoToFile=bSaveTimeDemoToFile;
	OldConsole.HistoryCur = HistoryCur;
	OldConsole.HistoryTop = HistoryTop;
	for (i = 0; i < MaxHistory; i++)
	    OldConsole.History[i] = History[i];

	//States:
	switch (MyState)
	{
		case 1:
			OldConsole.GotoState('UWindow');
			break;

		case 2:
			OldConsole.GotoState('Typing');
			break;

		default:
			OldConsole.GotoState('');
	}
	Viewport.Console = OldConsole; //Set Console back.
}

event bool KeyEvent( EInputKey Key, EInputAction Action, FLOAT Delta )
{
	local ManagerWindowStub ManagerMenu;
	local byte k;
	local bool CustomWin;

	if ( MyState != 0 )
	{
		if ( MyState == 1 && Key != IK_F9 ) //exception for screenshot.
			return WinKeyEvent(Key,Action,Delta);
		if ( MyState == 2 && Key==IK_Escape )
			return TypingKeyEvent(Key,Action,Delta);
	}
	//UT console:
	if ( Action != IST_Press )
		return false;
	if ( Key == IK_Escape )
	{
//		Log("**** ESC was pressed!!!!");
		SetOldConsole();
		Viewport.Console.KeyEvent( Key, Action, Delta );
		// Reset Player's Music Volume here.
		MyDEES.ClientResetMusicVol();
		return True;
	}
	if ( Key == SpeechKey )
	{
		if ( !bShowSpeech && !bTyping )
		{
			ShowSpeech();
			bQuickKeyEnable = True;
			LaunchUWindow();
		}
		return true;
	}
	k = Key;
	switch(Action)
	{
		case IST_Press:
			switch(k)
			{
				case EInputKey.IK_Escape:
					if ( bLocked )
						return true;
					bQuickKeyEnable = False;
					LaunchUWindow();
					return true;
				case ConsoleKey:
					if ( bLocked )
						return true;
					bQuickKeyEnable = True;
					LaunchUWindow();
					if ( !bShowConsole )
						ShowConsole();
					return true;
			}
			break;
	}
	if ( MyState == 2 ) //emulates global key event.
		return TypingKeyEvent(Key,Action,Delta);
	return False;
}

final function bool WinKeyEvent( EInputKey Key, EInputAction Action, FLOAT Delta )
{
	local byte k;

	//UTconsole:
	if ( Action == IST_Release && Key == SpeechKey )
	{
		if ( bShowSpeech )
			HideSpeech();
		return True;
	}
	if ( bShowSpeech && (SpeechWindow != None) )
	{
		//forward input to speech window
		if ( SpeechWindow.KeyEvent(Key, Action, Delta) )
			return true;
	}
	//WINDOWCONSOLE:
	k = Key;
	switch (Action)
	{
		case IST_Release:
			switch (k)
			{
				case EInputKey.IK_LeftMouse:
					if ( Root != None )
						Root.WindowEvent(WM_LMouseUp, None, MouseX, MouseY, k);
					break;
				case EInputKey.IK_RightMouse:
					if ( Root != None )
						Root.WindowEvent(WM_RMouseUp, None, MouseX, MouseY, k);
					break;
				case EInputKey.IK_MiddleMouse:
					if ( Root != None )
						Root.WindowEvent(WM_MMouseUp, None, MouseX, MouseY, k);
					break;
				default:
					if ( Root != None )
						Root.WindowEvent(WM_KeyUp, None, MouseX, MouseY, k);
					break;
			}
			break;
		case IST_Press:
			switch (k)
			{
				case ConsoleKey:
					if ( bShowConsole )
					{
						HideConsole();
						if ( bQuickKeyEnable )
							CloseUWindow();
					}
					else
					{
						if ( Root.bAllowConsole )
							ShowConsole();
						else
							Root.WindowEvent(WM_KeyDown, None, MouseX, MouseY, k);
					}
					break;
				case EInputKey.IK_Escape:
					if ( Root != None )
						Root.CloseActiveWindow();
					break;
				case EInputKey.IK_LeftMouse:
					if ( Root != None )
						Root.WindowEvent(WM_LMouseDown, None, MouseX, MouseY, k);
					break;
				case EInputKey.IK_RightMouse:
					if ( Root != None )
						Root.WindowEvent(WM_RMouseDown, None, MouseX, MouseY, k);
					break;
				case EInputKey.IK_MiddleMouse:
					if ( Root != None)
						Root.WindowEvent(WM_MMouseDown, None, MouseX, MouseY, k);
					break;
				default:
					if ( Root != None )
						Root.WindowEvent(WM_KeyDown, None, MouseX, MouseY, k);
					break;
			}
			break;
		case IST_Axis:
			switch (Key)
			{
				case IK_MouseX:
					MouseX = MouseX + (MouseScale * Delta);
					break;
				case IK_MouseY:
					MouseY = MouseY - (MouseScale * Delta);
					break;
			}
		default:
			break;
	}
	return true;
}
		 //typing state: (engine.console)
final function bool TypingKeyEvent( EInputKey Key, EInputAction Action, FLOAT Delta )
{
	local string Temp;

	bNoStuff = false;
	if ( Key == IK_Escape )
	{
		if ( Scrollback != 0 )
		{
			Scrollback = 0;
		}
		else
		if ( TypedStr != "" )
		{
			TypedStr = "";
		}
		else
		{
			ConsoleDest = 0.0;
			bTyping = false;
			Viewport.Actor.Typing(bTyping);
			MyState = 0;
		}
		Scrollback = 0;
	}
	else
	if ( Key == IK_Enter )
	{
		if ( Scrollback != 0 )
		{
			Scrollback = 0;
		}
		else
		{
			if ( TypedStr != "" )
			{
				// Print to console.
				if ( ConsoleLines != 0 )
					Message( None, "(>" @ TypedStr, 'Console' );
					// Update history buffer.
				History[HistoryCur++ % MaxHistory] = TypedStr;
				if ( HistoryCur > HistoryBot )
					HistoryBot++;
				if ( HistoryCur - HistoryTop >= MaxHistory )
					HistoryTop = HistoryCur - MaxHistory + 1;
				Temp = TypedStr;
				TypedStr = "";
				if ( !ConsoleCommand( Temp ) )
					Message( None, Localize("Errors","Exec","Core"), 'Console' );
				Message( None, "", 'Console' );
			}
			if ( ConsoleDest == 0.0 )
			{
				MyState = 0;
				bTyping = false;
				Viewport.Actor.Typing(false);
			}
			Scrollback = 0;
		}
	}
	else
	if ( Key == IK_Up )
	{
		if ( HistoryCur > HistoryTop )
		{
			History[HistoryCur % MaxHistory] = TypedStr;
			TypedStr = History[--HistoryCur % MaxHistory];
		}
		Scrollback=0;
	}
	else
	if ( Key == IK_Down )
	{
		History[HistoryCur % MaxHistory] = TypedStr;
		if ( HistoryCur < HistoryBot )
			TypedStr = History[++HistoryCur % MaxHistory];
		else
			TypedStr = "";
		Scrollback = 0;
	}
	else
	if ( Key == IK_PageUp )
	{
		if ( ++Scrollback >= MaxLines )
			Scrollback = MaxLines-1;
	}
	else
	if(  Key == IK_PageDown )
	{
		if ( --Scrollback < 0 )
			Scrollback = 0;
	}
	else
	if( Key == IK_Backspace || Key == IK_Left )
	{
		if ( Len(TypedStr) > 0 )
			TypedStr = Left(TypedStr,Len(TypedStr)-1);
		Scrollback = 0;
	}
	return true;
}

function ShowConsole()
{
	bShowConsole = true;
	if ( bCreatedRoot )
		ConsoleWindow.ShowWindow();
}

function HideConsole()
{
	ConsoleLines = 0;
	bShowConsole = false;
	if ( ConsoleWindow != None )
		ConsoleWindow.HideWindow();
}

//window state holders:
function LaunchUWindow()
{
	local int i;

	Viewport.bSuspendPrecaching = True;
	bUWindowActive = !bQuickKeyEnable;
	Viewport.bShowWindowsMouse = True;

	if ( bQuickKeyEnable )
		bNoDrawWorld = False;
	else
	{
		if ( Viewport.Actor.Level.NetMode == NM_Standalone )
			Viewport.Actor.SetPause( True );
		bNoDrawWorld = ShowDesktop;
	}
	if ( Root != None)
		Root.bWindowVisible = True;
	if ( MyState == 2 )
	{ //check typing!
		bTyping = false;
		Viewport.Actor.Typing(bTyping);
	}
	MyState = 1;
}

function CloseUWindow()
{
	if ( !bQuickKeyEnable )
		Viewport.Actor.SetPause( False );

	bNoDrawWorld = False;
	bQuickKeyEnable = False;
	bUWindowActive = False;
	Viewport.bShowWindowsMouse = False;

	if ( Root != None )
		Root.bWindowVisible = False;
	MyState = 0;
	Viewport.bSuspendPrecaching = False;
}

//State checks:
// Begin typing a command on the console.
exec function Type()
{
	TypedStr="";
	if ( MyState == 2 )
	{
		MyState = 0;
		bTyping = false;
		Viewport.Actor.Typing(bTyping);
		ConsoleDest = 0.0;
		return;
	}
	MyState = 2;
	bTyping = true;
	Viewport.Actor.Typing(bTyping);
}

exec function Talk()
{
	TypedStr="Say ";
	bNoStuff = true;
	MyState = 2;
	bTyping = true;
	Viewport.Actor.Typing(bTyping);
}

exec function TeamTalk()
{
	TypedStr="TeamSay ";
	bNoStuff = true;
	MyState = 2;
	bTyping = true;
	Viewport.Actor.Typing(bTyping);
}

exec function ViewUp();

exec function ViewDown();

