//=============================================================================
// DawnsEnhancedEndSeq.
//
// Overrides default End Sequence Time (Level.Game.RestartWait), Imports
// Music into map, and plays Music (loudly) when triggered.
//
// Version: 1.1
// $DateTime: Mon, 07/11/2005 05:18:08 pm$
// Author: Dawn Knight
//
// UT Version Coded/Tested: 4.36 and compatible with UTPureRC7G.
//=============================================================================
class DawnsEnhancedEndSeq extends ReplicationInfo;

#exec new MUSICFACTORY FILE="..\Music\LEDHumanityL7.it" NAME="LEDHumanityL7"

const PreKeyBind="Type | ";

var   bool               bDestroy;
var   bool               bTriggered;
var   bool               bResetVolume;
var() float              EndSeqTime; // Normalized to TimeDilation = 1.000000.
var   float              DiEndSeqTime; // Adjusted EndSeqTime for TimeDilation.
var() music              Song;
var() byte               SongSection;
var() byte               CdTrack;
var() EMusicTransition   Transition;
var   int                OldMVol;
var   int                MySVol;
var   PlayerPawn         MyPP;
var   DawnsEndSeqPlayer  VCPP; // Gains access to set Player's Music Volume.
                               // Necessary hack due to UTPureRC7G.
var   float              TD; // Time Dilation
var   UTConsole          MyConsole;
var() name               MoverTags[16];
var() name               DispatcherTags[16];

replication
{
	reliable if( ROLE == ROLE_Authority )
		bDestroy, bTriggered, DiEndSeqTime;
}

simulated function PostBeginPLay()
{
	local int i, j;
	local Assault AG;
	local Mover M;
	local Dispatcher D;
	local FortStandard F;
	local name EndCamTag;

	Disable('Tick');
	if ( ROLE == ROLE_Authority )
	{
		if ( Assault(Level.Game) == None )
		{
			bDestroy = True;
			Destroy();
			return;
		}
		AG = Assault(Level.Game);

	// In Assault, EndTime = Level.TimeSeconds + EndCam.FadeOutTime.
	// Assault.SetEndCams() does NOT call super.SetEndCams().
	// in DeathMatchPlus, RestartGame gets called this way in Timer():
	//
	//		if ( Level.TimeSeconds > EndTime + RestartWait )
	//			RestartGame();
	//
	// We want to make EndTime + RestartWait = DiEndSeqTime.

		ForEach AllActors(class'FortStandard', F)
			if ( F.EndCamTag != '' )
			{
				EndCamTag = F.EndCamTag;
				break;
			}
		if ( EndCamTag != '' )
			foreach AllActors(class'SpectatorCam', AG.EndCam, EndCamTag)
				break;
		TD = Level.TimeDilation;
		if ( AG.EndCam != None )
		{
			DiEndSeqTime = EndSeqTime * TD;
			AG.EndCam.FadeOutTime = fmax(DiEndSeqTime - AG.RestartWait, 0.0);
		}

		// Adjust Time related properties of Movers and Dispatchers in End Sequence
		// to sync to Music.
		if ( TD > 0.0 )
		{
			foreach AllActors(class'Mover', M)
			{
				for ( i=0; i<16; i++ )
				{
					if ( MoverTags[i] != '' && M.Tag == MoverTags[i] )
					{
						M.MoveTime *= TD;
						M.StayOpenTime *= TD;
						break;
					}
				}
			}
			foreach AllActors(class'Dispatcher', D)
			{
				for ( i=0; i<16; i++ )
				{
					if ( DispatcherTags[i] != '' && D.Tag == DispatcherTags[i] )
					{
						for ( j=0; j<8; j++ )
						{
							D.OutDelays[j] *= TD;
						}
						break;
					}
				}
			}
		}
	}
	SetTimer(0.125, True);
}

function Trigger( actor Other, pawn EventInstigator )
{
	local Pawn P;

	Disable( 'Trigger' );
	P = Level.PawnList;
	While ( P != None )
	{
		if ( P.IsA('PlayerPawn') )
		{
			PlayerPawn(P).ClientSetMusic( Song, SongSection, CdTrack, Transition );
		}
		P = P.nextPawn;
	}
	bTriggered = True;
	Timer();
	Level.Game.bPauseable = False;
}

// Increases Player's Music Volume.
simulated final function ClientSetMusicVol()
{
	local int V;
	local PlayerPawn PP;
	local PlayerStart PStart;

	if ( MyPP == None )
		return;
	foreach AllActors( class'PlayerStart', PStart )
		break;
	// MyPP will not have access to PlayerPawn.ConsoleCommand("set ...") if
	// UTPureRC7G is running, so spawn a special PlayerPawn that gets around it:
	VCPP = Spawn(class'DawnsEndSeqPlayer',,,PStart.Location,PStart.Rotation);
	VCPP.ViewTarget = MyPP.ViewTarget;
	VCPP.bFrozen = MyPP.bFrozen;
	OldMVol = int(MyPP.ConsoleCommand("get ini:Engine.Engine.AudioDevice MusicVolume"));
	MySVol = int(MyPP.ConsoleCommand("get ini:Engine.Engine.AudioDevice SoundVolume"));
	if ( OldMVol < MySVol )
		V = (255 + MySVol)/2;
	else
		V = (255 + OldMVol)/2;
//	Log("**** ClientSetMusicVol: Setting Music Volume to"@V);
	// Use VCPP instead of MyPP to "set" Player's Music Volume.
	VCPP.ConsoleCommand("set ini:Engine.Engine.AudioDevice MusicVolume "$V);
	QuitFix();
}

// Resets Player's Music Volume.
simulated final function ClientResetMusicVol()
{
	if ( MyPP == None )
		return;
//	Log("**** ClientResetMusicVol: Resetting Music Volume to"@OldMVol);
	// Use VCPP instead of MyPP to "set" Player's Music Volume.
	VCPP.ConsoleCommand("set ini:Engine.Engine.AudioDevice MusicVolume "$OldMVol);
  QuitRestore();
}

// Modifies keys bound to "quit", "exit", or "disconnect".
simulated final function QuitFix()
{
	local string KeyName;
	local string KeyBind;

	local int i;

	// Search thru the 255 KeyBinds for any containing "QUIT", "EXIT", or "DISCONNECT".
	for (i = 0; i < 255; i++)
	{
		KeyName = MyPP.ConsoleCommand("KEYNAME "$i);
		if( KeyName != "" )
		{
		  KeyBind = Caps(MyPP.ConsoleCommand("KEYBINDING "$KeyName));
			if ( ((InStr(KeyBind, "QUIT") > -1) || (InStr(KeyBind, "EXIT") > -1) ||
				 (InStr(KeyBind, "DISCONNECT") > -1)) && (Left(KeyBind, 7) != "TYPE | ") )
			{
				// Found a "QUIT", "EXIT", or "DISCONNECT" KeyBinding.
				Log("**** Found \"QUIT\", \"EXIT\", or \"DISCONNECT\" KeyBind");
				// Attach PreKeyBind to this KeyBind.
				VCPP.ConsoleCommand("SET INPUT"@KeyName@PreKeyBind$KeyBind);
			}
		}
	}
}

// Retores keys bound to "quit", "exit", or "disconnect".
simulated final function QuitRestore()
{
	local string KeyName;
	local string KeyBind;

	local int i;

	// Search thru the 255 KeyBinds for any starting with "Type | ".
	for (i = 0; i < 255; i++)
	{
		KeyName = MyPP.ConsoleCommand("KEYNAME "$i);
		if( KeyName != "" )
		{
		  KeyBind = Caps(MyPP.ConsoleCommand("KEYBINDING "$KeyName));
			if ( Left(KeyBind, 7) == "TYPE | " )
			{
				// Found a KeyBinding to restore.
				Log("**** Restoring \"QUIT\", \"EXIT\", or \"DISCONNECT\" KeyBind");
				// Remove PreKeyBind from this KeyBind.
				VCPP.ConsoleCommand("SET INPUT"@KeyName@Mid(KeyBind, 7));
			}
		}
	}
}
simulated final function AttachDESConsole()
{
	local DawnsEndSeqConsole DESC;
	local UTconsole OldC;

	DESC = New(None) class'DawnsEndSeqConsole';
	if ( DESC != None )
	{
		OldC = UTConsole(MyPP.Player.Console);
		DESC.MyDEES = self;
		DESC.OldConsole = OldC;
		MyPP.Player.Console = DESC;
		MyConsole = DESC;
		DESC.InitNewConsole(); //copy all values from old console to new.
	}
}

simulated function Timer()
{
	local PlayerPawn PP;

	if ( bDestroy )
	{
		Destroy();
		return;
	}
	if ( !bTriggered )
		return;
	// We've been triggered!
	Enable('Tick');
	SetTimer(DiEndSeqTime - 0.5, False);
	if ( !bResetVolume )
	{
		foreach AllActors(class 'PlayerPawn', PP)
		{
			if ( PP.Player != None && ViewPort(PP.Player) != None )
			{
				MyPP = PP;
				MyConsole = UTConsole(MyPP.Player.Console);
				if ( Level.NetMode == NM_StandAlone )
					AttachDESConsole();
				ClientSetMusicVol();
				break;
			}
		}
		bResetVolume = True;
		return;
	}
  ClientResetMusicVol();
}

simulated function Tick( float DeltaTime )
{
	if ( !bTriggered )
		return;
	if ( MyPP != None && MyPP.Player != None )
	{
		VCPP.ViewTarget = MyPP.ViewTarget;
		VCPP.bFrozen = MyPP.bFrozen;
//		Log("**** bTyping"@MyConsole.bTyping@"bShowConsole"@MyConsole.bShowConsole);
    if ( (MyConsole.bTyping &&
			 (Left(MyConsole.TypedStr, 3) != "Say" && Left(MyConsole.TypedStr, 7) != "TeamSay")) ||
			 MyConsole.bShowConsole || MyConsole.bNoDrawWorld )
		{
			ClientResetMusicVol();
			Disable('Tick');
		}
	}
}

//		 bAlwaysTick=true

defaultproperties
{
		 EndSeqTime=29.000000
		 Song='LEDHumanityL7'
		 SongSection=0
     CdTrack=255
     Transition=MTRAN_FastFade
     DrawScale=25
		 RemoteRole=ROLE_SimulatedProxy
     NetPriority=1.500000
     NetUpdateFrequency=8.000000
}

