//=============================================================================
// DawnsProjectileFactory.
//
// Spawns Projectiles complete with effects.
//
// Version: 1.0
// $DateTime: Fri, 07/08/2005 09:48:05 pm$
// Author: Dawn Knight
//
// UT Version Coded/Tested: 4.36.
//=============================================================================
class DawnsProjectileFactory extends Keypoint;

var() class<Projectile> prototype; 	// the template Projectile class.
var() class<ExplosionChain> protoEffect; 	// the template Projectile class effect.
var() int maxitems;	// max number of items from this factory at any time
var	  int	numitems;	// current number of items from this factory
var() int capacity;		// max number of items ever buildable (-1 = no limit)
var() float interval;	// average time interval between spawnings
var() name	itemtag;	// tag given to items produced at this factory
var() bool	bFalling;	// non-pawn items spawned should be set to falling
var() float MomentumTransfer;
var() float Damage;
var() float Size;
var() float Delaytime;

var() enum EDistribution
{
	DIST_Constant,
	DIST_Uniform,
	DIST_Gaussian
}
timeDistribution;

function PostBeginPlay()
{
	Super.PostBeginPlay();
	numitems = 0;
	if (itemtag == '')
		itemtag = 'Dawns';
}


function StartBuilding()
{
}

auto State Waiting
{
	function Trigger( actor Other, pawn EventInstigator )
	{
		local Actor A;

		if ( Event != '' )
			ForEach AllActors( class 'Actor', A, Event)
				A.Trigger(self, EventInstigator);
		GotoState('Spawning');
	}

	function Touch(Actor Other)
	{
	}
}

function bool Create()
{
	local actor temp, A;
	local rotator newRot;
	local ExplosionChain tempEffect;

	temp = Spawn(prototype);
	if (temp == None)
		return false;
	newRot = rot(0,0,0);
	newRot.yaw = rotation.yaw;
	temp.SetRotation(newRot);
	temp.event = tag;
	temp.tag = itemtag;

	if( event != '' )
		foreach AllActors( class 'Actor', A, Event )
			A.Trigger( Self, Instigator );
	if ( bFalling )
		temp.SetPhysics(PHYS_Falling);

	tempEffect = Spawn(protoEffect);
	tempEffect.MomentumTransfer = MomentumTransfer;
	tempEffect.Damage = Damage;
	tempEffect.Size = Size;
	tempEffect.Delaytime = DelayTime;
	tempEffect.GotoState('Exploding');
	return true;
}

state Spawning
{
	function UnTouch(Actor Other)
	{
	}

	function Trigger(actor Other, pawn EventInstigator)
	{
		//only if Other is from this factory
		if ( Other.class != prototype )
			return;

		numitems--;
		if (numitems < maxitems)
			StartBuilding();
	}

	function bool trySpawn()
	{
		local bool done;

		done = false;
		if ( Create() )
		{
			done = true;
			capacity--;
			numitems++;
			if (capacity == 0)
				GotoState('Finished');
		}
		return done;
	}

	function Timer()
	{
		local int start;

		if (numitems < maxitems)
		{
			trySpawn();
		}

		if (numitems < maxitems)
			StartBuilding();
	}

	Function StartBuilding()
	{
		local float nextTime;
		if (timeDistribution == DIST_Constant)
			nextTime = interval;
		else if (timeDistribution == DIST_Uniform)
			nextTime = 2 * FRand() * interval;
		else //timeDistribution is gaussian
			nextTime = 0.5 * (FRand() + FRand() + FRand() + FRand()) * interval;

		if (capacity > 0)
			SetTimer(nextTime, false);
	}

	function BeginState()
	{
	}

Begin:
	Timer();
}

state Finished
{
}

defaultproperties
{
     MaxItems=1
     capacity=1000000
     interval=1.000000
     bFalling=True
     bStatic=False
     bCollideActors=True
}
