//=============================================================================
// DawnsRecoilMover.
//
// Provides a realistic recoil action for cannons and large guns by using
// different Opening and Closing MoveTimes.
//
// Version: 1.0
// $DateTime: Thu, 07/07/2005 06:17:08 pm$
// Author: Dawn Knight
//
// UT Version Coded/Tested: 4.36.
//=============================================================================
class DawnsRecoilMover extends Mover;

var() float      OpenTime;    // Time to spend moving from keyframe 0 to 1.
var() float      CloseTime;   // Time to spend moving back from keyframe 1 to 0.

// Open the mover.
function DoOpen()
{
	bOpening = true;
	bDelaying = false;
	InterpolateTo( 1, OpenTime );
	PlaySound( OpeningSound, SLOT_None );
	AmbientSound = MoveAmbientSound;
}

// Close the mover.
function DoClose()
{
	local actor A;

	bOpening = false;
	bDelaying = false;
	InterpolateTo( Max(0,KeyNum-1), CloseTime );
	PlaySound( ClosingSound, SLOT_None );
	if( Event != '' )
		foreach AllActors( class 'Actor', A, Event )
			A.UnTrigger( Self, Instigator );
	AmbientSound = MoveAmbientSound;
}

defaultproperties
{
     MoverEncroachType=ME_CrushWhenEncroach
     OpenTime=0.200000
     CloseTime=1.200000
     StayOpenTime=0.050000
}
